package org.apache.commons.lang3.builder;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import org.apache.commons.lang3.ArraySorter;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.Validate;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringExclude;
import org.apache.commons.lang3.builder.ToStringStyle;
import org.apache.commons.lang3.builder.ToStringSummary;

public class ReflectionToStringBuilder extends ToStringBuilder {
  private boolean appendStatics;
  
  private boolean appendTransients;
  
  private boolean excludeNullValues;
  
  protected String[] excludeFieldNames;
  
  private Class<?> upToClass;
  
  public static String toString(Object object) {
    return toString(object, (ToStringStyle)null, false, false, (Class<? super Object>)null);
  }
  
  public static String toString(Object object, ToStringStyle style) {
    return toString(object, style, false, false, (Class<? super Object>)null);
  }
  
  public static String toString(Object object, ToStringStyle style, boolean outputTransients) {
    return toString(object, style, outputTransients, false, (Class<? super Object>)null);
  }
  
  public static String toString(Object object, ToStringStyle style, boolean outputTransients, boolean outputStatics) {
    return toString(object, style, outputTransients, outputStatics, (Class<? super Object>)null);
  }
  
  public static <T> String toString(T object, ToStringStyle style, boolean outputTransients, boolean outputStatics, Class<? super T> reflectUpToClass) {
    return (new org.apache.commons.lang3.builder.ReflectionToStringBuilder(object, style, null, reflectUpToClass, outputTransients, outputStatics))
      .toString();
  }
  
  public static <T> String toString(T object, ToStringStyle style, boolean outputTransients, boolean outputStatics, boolean excludeNullValues, Class<? super T> reflectUpToClass) {
    return (new org.apache.commons.lang3.builder.ReflectionToStringBuilder(object, style, null, reflectUpToClass, outputTransients, outputStatics, excludeNullValues))
      .toString();
  }
  
  public static String toStringExclude(Object object, Collection<String> excludeFieldNames) {
    return toStringExclude(object, toNoNullStringArray(excludeFieldNames));
  }
  
  static String[] toNoNullStringArray(Collection<String> collection) {
    if (collection == null)
      return ArrayUtils.EMPTY_STRING_ARRAY; 
    return toNoNullStringArray(collection.toArray());
  }
  
  static String[] toNoNullStringArray(Object[] array) {
    List<String> list = new ArrayList<>(array.length);
    for (Object e : array) {
      if (e != null)
        list.add(e.toString()); 
    } 
    return list.<String>toArray(ArrayUtils.EMPTY_STRING_ARRAY);
  }
  
  public static String toStringExclude(Object object, String... excludeFieldNames) {
    return (new org.apache.commons.lang3.builder.ReflectionToStringBuilder(object)).setExcludeFieldNames(excludeFieldNames).toString();
  }
  
  private static Object checkNotNull(Object obj) {
    return Validate.notNull(obj, "obj", new Object[0]);
  }
  
  public ReflectionToStringBuilder(Object object) {
    super(checkNotNull(object));
  }
  
  public ReflectionToStringBuilder(Object object, ToStringStyle style) {
    super(checkNotNull(object), style);
  }
  
  public ReflectionToStringBuilder(Object object, ToStringStyle style, StringBuffer buffer) {
    super(checkNotNull(object), style, buffer);
  }
  
  public <T> ReflectionToStringBuilder(T object, ToStringStyle style, StringBuffer buffer, Class<? super T> reflectUpToClass, boolean outputTransients, boolean outputStatics) {
    super(checkNotNull(object), style, buffer);
    setUpToClass(reflectUpToClass);
    setAppendTransients(outputTransients);
    setAppendStatics(outputStatics);
  }
  
  public <T> ReflectionToStringBuilder(T object, ToStringStyle style, StringBuffer buffer, Class<? super T> reflectUpToClass, boolean outputTransients, boolean outputStatics, boolean excludeNullValues) {
    super(checkNotNull(object), style, buffer);
    setUpToClass(reflectUpToClass);
    setAppendTransients(outputTransients);
    setAppendStatics(outputStatics);
    setExcludeNullValues(excludeNullValues);
  }
  
  protected boolean accept(Field field) {
    if (field.getName().indexOf('$') != -1)
      return false; 
    if (Modifier.isTransient(field.getModifiers()) && !isAppendTransients())
      return false; 
    if (Modifier.isStatic(field.getModifiers()) && !isAppendStatics())
      return false; 
    if (this.excludeFieldNames != null && 
      Arrays.binarySearch((Object[])this.excludeFieldNames, field.getName()) >= 0)
      return false; 
    return !field.isAnnotationPresent((Class)ToStringExclude.class);
  }
  
  protected void appendFieldsIn(Class<?> clazz) {
    if (clazz.isArray()) {
      reflectionAppendArray(getObject());
      return;
    } 
    Field[] fields = ArraySorter.<Field>sort(clazz.getDeclaredFields(), Comparator.comparing(Field::getName));
    AccessibleObject.setAccessible((AccessibleObject[])fields, true);
    for (Field field : fields) {
      String fieldName = field.getName();
      if (accept(field))
        try {
          Object fieldValue = getValue(field);
          if (!this.excludeNullValues || fieldValue != null)
            append(fieldName, fieldValue, !field.isAnnotationPresent((Class)ToStringSummary.class)); 
        } catch (IllegalAccessException ex) {
          throw new InternalError("Unexpected IllegalAccessException: " + ex.getMessage());
        }  
    } 
  }
  
  public String[] getExcludeFieldNames() {
    return (String[])this.excludeFieldNames.clone();
  }
  
  public Class<?> getUpToClass() {
    return this.upToClass;
  }
  
  protected Object getValue(Field field) throws IllegalAccessException {
    return field.get(getObject());
  }
  
  public boolean isAppendStatics() {
    return this.appendStatics;
  }
  
  public boolean isAppendTransients() {
    return this.appendTransients;
  }
  
  public boolean isExcludeNullValues() {
    return this.excludeNullValues;
  }
  
  public org.apache.commons.lang3.builder.ReflectionToStringBuilder reflectionAppendArray(Object array) {
    getStyle().reflectionAppendArrayDetail(getStringBuffer(), null, array);
    return this;
  }
  
  public void setAppendStatics(boolean appendStatics) {
    this.appendStatics = appendStatics;
  }
  
  public void setAppendTransients(boolean appendTransients) {
    this.appendTransients = appendTransients;
  }
  
  public void setExcludeNullValues(boolean excludeNullValues) {
    this.excludeNullValues = excludeNullValues;
  }
  
  public org.apache.commons.lang3.builder.ReflectionToStringBuilder setExcludeFieldNames(String... excludeFieldNamesParam) {
    if (excludeFieldNamesParam == null) {
      this.excludeFieldNames = null;
    } else {
      this.excludeFieldNames = ArraySorter.<String>sort(toNoNullStringArray((Object[])excludeFieldNamesParam));
    } 
    return this;
  }
  
  public void setUpToClass(Class<?> clazz) {
    if (clazz != null) {
      Object object = getObject();
      if (object != null && !clazz.isInstance(object))
        throw new IllegalArgumentException("Specified class is not a superclass of the object"); 
    } 
    this.upToClass = clazz;
  }
  
  public String toString() {
    if (getObject() == null)
      return getStyle().getNullText(); 
    Class<?> clazz = getObject().getClass();
    appendFieldsIn(clazz);
    while (clazz.getSuperclass() != null && clazz != getUpToClass()) {
      clazz = clazz.getSuperclass();
      appendFieldsIn(clazz);
    } 
    return super.toString();
  }
}
